# generates sparse (spam package format) MRF precision matrices for CAR and TPS models; note that the neighborhood info in the 'graph' and 'graphCats' files must already exist
graphRead <- function(mrfType, m1, m2, distFileName = NULL, distances = FALSE){
  if(mrfType == 'bin'){
    graphFileName <- paste('graphBin', m1, 'x', m2, '.csv', sep = '')
  } else{
    graphFileName <- paste('graphTps', m1, 'x', m2, '.csv', sep = '')
    catFile <- paste('graphCatsTps', m1, 'x', m2, '.csv', sep = '')
  }

  if(mrfType == 'tps'){
    cats <- scan(catFile)
  }

  if(mrfType == 'tps'){
    map <- c(rep(0, 4), 4, 0, 10, 11, 0, 18, 19, 20)
  } else{
    map <- 1:4
  }
  Q <- as.spam(diag(rep(1, 2)))
  graphVals <- as.integer(scan(graphFileName))
  Q@dimension <- c(graphVals[1], graphVals[1])
  dim <- Q@dimension[1]
  rowpointers <- rep(0, dim+1)
  
  graph <- list()
  graph$n <- graphVals[1]
  graph$cells <- NULL
  graph$nnbs <- NULL
  graph$nbs <- NULL
  graph$ptr <- NULL
  counter <- 1
  nbCounter <- nbCounterGraph <- 1
  entries <- colindices <- NULL
  
  for(node in 0:(m-1)){
    nodex <- node%%m1
    nodey <- floor(node/m1)
    counter <- counter+1
    graph$cells[node+1] <- graphVals[counter]
    counter <- counter+1
    nnbs <- graph$nnbs[node+1] <- graphVals[counter]
    rowpointers[node+1] <- nbCounter
    graph$nbPtr[node+1] <- nbCounterGraph
    
    if(nnbs){
      graph$nbs[(nbCounterGraph):(nbCounterGraph+graph$nnbs[node+1]-1)] <- graphVals[(counter+1):(counter+graph$nnbs[node+1])]
      nbCounterGraph <- nbCounterGraph+graph$nnbs[node+1]
      nbs <- graphVals[(counter+1):(counter+nnbs)]
      nbs <- sort(c(nbs, node))
      nbCounter <- nbCounter+nnbs+1
    } else{
      nbs <- node
    }
    counter <- counter+nnbs
    
    nnodex <- nbs%%m1
    nnodey <- floor(nbs/m1)
    tmpent <- NULL
    for(nnodeIndex in 1:length(nbs)){
      nnode <- nbs[nnodeIndex]
      if(node == nnode){
        value <- map[nnbs]
      } else{
        if(mrfType == 'tps'){
          if(abs(nodex-nnodex[nnodeIndex]) == 2 | abs(nodey-nnodey[nnodeIndex]) == 2){
            value <- 1
          } else{
            if(abs(nodex-nnodex[nnodeIndex]) == 1 & abs(nodey-nnodey[nnodeIndex]) == 1){
              value <- 2
            }
            else{
              value <- -8
              if(cats[node+1] == 2 | cats[node+1] == 3){
                value <- -6
              }
              if(cats[node+1] == 1){
                value <- -4
              }
              if((cats[node+1] == 3 & cats[nnode+1] == 1) | (cats[node+1] == 4 & cats[nnode+1] == 3) |(cats[node+1] == 5 & cats[nnode+1] == 2)){
                value <- value+2
              }
            }
          }
        } else{
          if(abs(nodex-nnodex[nnodeIndex])+abs(nodey-nnodey[nnodeIndex]) == 1){
            value <- -1
          }
        }
      }
      tmpent <- c(tmpent, value)
    }
    entries <- c(entries, tmpent)
    colindices <- c(colindices, nbs+1)
  }
  rowpointers[dim+1] <- length(entries)+1
  Q@rowpointers <- as.integer(rowpointers)
  Q@colindices <- as.integer(colindices)
  Q@entries <- entries
  return(list(Q, graph))
}

# makes individual panels of Figs. 5a, 6a, 8a
plotFun  <-  function(sseOracleObj, sseGPobj, sseOracleObj2 = NULL, sseGPobj2 = NULL, k, mm, type){
  if(k == 2) xaxt <- "s" else xaxt  <-  "n"
  if(mm == 1) yaxt  <-  "s" else yaxt  <-  "n"
  rg <- c(-1, 2)
  if(type == 'sparse'){
    plot(rhovals, log2(colMeans(sseOracleObj[, 1, k, , mm]/sseOracleObj[, 2, k, , mm])), pch = 16, cex = .7, ylim = rg, type = 'b', xlab = '', ylab = '', log = 'x',  xaxt  =  xaxt,  yaxt = yaxt,  cex.axis = 1.1)
    lines(rhovals, log2(colMeans(sseGPobj[, k, , mm]/sseOracleObj[, 2, k, , mm])), col = 'blue', type = 'b', pch = 16, cex = .7)
  } else{  # 'areal'
    plot(rhovals, log2(sseOracleObj[1, k, , mm]/sseOracleObj[2, k, , mm]), pch = 16, cex = .7, ylim = rg, type = 'b', xlab = '', ylab = '', log = 'x',  xaxt  =  xaxt,  yaxt = yaxt,  cex.axis = 1.1)
    lines(rhovals, log2(sseGPobj[k, , mm]/sseOracleObj[2, k, , mm]), col = 'blue', type = 'b', pch = 16, cex = .7)
  }
  if(type == 'sparse'){
    lines(rhovals, log2(colMeans(sseOracleObj2[, 1, k, , mm]/sseOracleObj2[, 2, k, , mm])), col = 'black', type = 'b', pch = 16, cex = .7, lty = 2)
    lines(rhovals, log2(colMeans(sseGPobj2[, k, , mm]/sseOracleObj2[, 2, k, , mm])), col = 'blue', type = 'b', pch = 16, cex = .7, lty = 2)
  } else{ # 'areal'
    lines(rhovals, log2(sseOracleObj2[1, k, , mm]/sseOracleObj2[2, k, , mm]), col = 'black', type = 'b', pch = 16, cex = .7, lty = 4)
    lines(rhovals, log2(sseGPobj2[k, , mm]/sseOracleObj2[2, k, , mm]), col = 'blue', type = 'b', pch = 16, cex = .7, lty = 4)
  }
  abline(h = log2(.9), lty = 1, col = 'lightgray')
  if(k == 1 & mm == 1 ){
    if(type == 'sparse'){
      legend(.004, 1.35, bty = 'n', lty = c(1, 2, 1, 2), col = c('black', 'black', 'blue', 'blue'), legend = c('ICAR,  uniform sampling', 'ICAR,  clustered sampling', 'GP,  uniform sampling', 'GP,  clustered sampling'), cex = .9)
    } else{
      if(type == 'areal'){
        legend(.004, 1.5, bty = 'n', lty = c(1, 4, 1, 4), col = c('black', 'black', 'blue', 'blue'), legend = c('ICAR,  full grid', 'ICAR,  obs. grid', 'GP,  full grid', 'GP,  obs. grid'), cex = .9)
      } else{
        legend(.004, 1.5, bty = 'n', lty = c(1, 1), col = c('black', 'blue'), legend = c('ICAR', 'GP'), cex = .9)      
      }
    }
  }
  par(new = TRUE)
  if(type == 'sparse'){
    plot(rhovals, colMeans(sseOracleObj[, 2, k, , mm]), pch = 16, cex = .7, col = 'red', xaxt = 'n', yaxt = 'n', xlab = '', ylab = '', ylim = c(1, 11000), log = 'xy', type = 'b')
    lines(rhovals, colMeans(sseOracleObj2[, 2, k, , mm]), pch = 16, cex = .7, col = 'red', lty  =  2, type = 'b')
  } else{
    if(type == 'areal'){
      plot(rhovals, sseOracleObj[2, k, , mm], pch = 16, cex = .7, col = 'red', xaxt = 'n', yaxt = 'n', xlab = '', ylab = '', ylim = c(1, 11000), log = 'xy', type = 'b')
      tmp <- sseOracleObj2[2, k, , mm]
      tmp[tmp<1] <- .7
      lines(rhovals, tmp, pch = 16, cex = .7, col = 'red', lty = 4)
    } else{
      plot(rhovals, sseOracleObj[2, k, , mm], pch = 16, cex = .7, col = 'red', xaxt = 'n', yaxt = 'n', xlab = '', ylab = '', ylim = c(1, 11000), log = 'xy', type = 'b')
    }
  }
  if(mm  ==  4)
    axis(4, col = 'red', col.lab = 'red', col.axis = 'red',  cex.axis = 1.1)
  if(mm == 1 && k == 2)
    title(ylab = "log2 of SSE relative to TPS-MRF",  outer = TRUE,  line = 1.45,  cex.lab = 1.2,  font.lab = 1)
  if(k == 2 && mm == 1)
    title(xlab = expression(rho),  outer = TRUE,  line = 1.5,  font.lab = 1,  cex.lab = 1.2)
}

# generates points from a Poisson cluster process
# modified based on Diggle 'Statistical Analysis of Spatial Point Patterns' book p. 66 to randomly assign events to parents,  conditioning on n
pcpGen <- function(n = 100, parentIntens = 25, sd = .05){
  locs <- matrix(NA,  nr = n, nc = 2)
  nParents <- 0
  while(nParents<1){
    nParents <- rpois(1, parentIntens)
  }
  parents <- cbind(runif(nParents), runif(nParents))
  count <- 1
  while(count<n+1){
    wh <- sample(1:nParents, 1)
    locs[count, ] <- rnorm(2, 0, sd)+parents[wh, ]
    if(max(locs[count, ])>1 | min(locs[count, ])<0){
    } else{
      count <- count+1
    }
  }
  return(list(locs = locs, parents = parents))
}

